
require 'timeout'
require 'email/emaimessages'

class Account
  attr_accessor :conn, :conf, :folders, :messages, :threads

  def initialize(conf)
    @conf = conf
    @conn = nil
    @messages = nil
    @folders = []
    @folder  = nil
  end

  def get_messages
    @subjects = []
    parts = ["UID", "ENVELOPE","RFC822.SIZE", "FLAGS"]
    begin
      messages = uid_fetch(1..-1, parts)
    rescue
      $logger.error "EXCEPTION in get_messages: #{$!}"
      messages = nil
    end
    if messages
      @messages = Messages.new(self)
      messages.each { |m|
        @messages[m.seqno] = Message.new(self) unless @messages[m.seqno]
        @messages[m.seqno].seqno               = m.seqno
        @messages[m.seqno].envelope            = m.attr["ENVELOPE"]
        @messages[m.seqno].envelope.subject = "(no subject)" if @messages[m.seqno].envelope.subject.nil?
        @messages[m.seqno].data["UID"]         = m.attr["UID"]
        @messages[m.seqno].data["RFC822.SIZE"] = m.attr["RFC822.SIZE"]
        @messages[m.seqno].data["FLAGS"]       = m.attr["FLAGS"]
        @messages.by_uid[m.attr["UID"]] = @messages[m.seqno]
      }
    else
      @messages = nil
    end
    @messages
  end


  def get_threads_info
    begin
      @conn.thread()
    rescue
      $logger.error "EXCEPTION in get_threads_info #{$!}"
    end
  end

  def get_body(message)
    if message
      begin
        res = uid_fetch(message.uid, "BODY[TEXT]")
        if res && res[0]
          message.body = res[0].attr["BODY[TEXT]"] or ""
        else
          message.body = "(Error Fetching Body)"
        end
        # XXX why did i put that here ?
        # if res && res[0]
        #  res = uid_fetch(message.uid, "FLAGS")
        #  message.data["FLAGS"] = res[0].attr["FLAGS"]
        return message.body
      rescue TimeoutError
        $logger.warn "TIMEOUT: $!"
        return "ERROR in get_body"
      end
    end
  end

  def get_header(message)
    part = "BODY[HEADER]"
    if message
      result = uid_fetch(message.uid, part)[0]
      header = result.attr[part]
      return header
    end
  end

  def mark(message, f)
    if message.kind_of?(Array) then
      res = @conn.uid_store(message.collect {|m| m.uid }, "+FLAGS", f)
      # XXX can we assume the arrays to be aligned?
      res.each_index { |i| message[i].data["FLAGS"] = res[i].attr["FLAGS"] }
    else
      res = @conn.uid_store([message.uid], "+FLAGS", f)
      if res && res[0] && message.data then
        message.data["FLAGS"] = res[0].attr["FLAGS"]
      end
    end
  end

  def unmark(message, f)
    res = @conn.uid_store([message.uid], "-FLAGS", f)
    if res && res[0] && message.data then
      message.data["FLAGS"] = res[0].attr["FLAGS"]
    end
  end

  def num_msgs(mbox = nil)
    begin
      @conn.imap.status(mbox,["MESSAGES","RECENT"])
    rescue
      return {"MESSAGES" => 0, "RECENT" => 0}
    end
  end

  def open(folder = @conf["Default Folder"])
    login if @conn == nil
    @folders = rec_list("").flatten
  end

  def rec_list(folder)
    #@conn.list("user" + "." + @conf["user"], '*') # .collect {|m| m}
    folders = @conn.list(folder, '%')
    folders.collect! { |f|
      if f.name != "INBOX.INBOX" && (f.attr.index(:Children) || f.attr.index(:Haschildren))  then
        [Folder.new(self,f)] + rec_list(f.name + ".")
      else
        Folder.new(self,f)
      end
    }
  end

  def select(folder)
    @folder.messages = @messages if @folder # save messages
    @folder = folder
    # did not spend time to think about synchronization
    # so skip this for now
    #@messages = @folder.messages            # restore messages
    begin
      @conn.select(folder.name)
    rescue Net::IMAP::NoResponseError
      return nil
    end
  end

  def uid_fetch(key, part) # raises TimeoutError
    @conn.uid_fetch(key, part)
  end

  def create(*args)
    @conn.create(*args)
  end

  def delete(*args)
    @conn.delete(*args)
  end

  def expunge()
    @conn.expunge()
    get_messages()
  end

  def thread()
    @conn.thread()
  end

  def login
    begin
      @conf["protocol"] == "imaps" ? ssl = true : ssl = false
      @conn = EmaiImap.new(@conf["user"], @conf["password"], @conf["server"], @conf["port"], ssl )
      @conn
    rescue
      $logger.error <<EOF
      Seems like we encountered an error :/
      while loggin into your account:
        #{$!}
EOF
      nil
    end
  end

  def name
    @conf["name"]
  end
end

class Folder
  attr_accessor :account, :messages, :num_msgs, :data, :num_recent
    
  def initialize(account,folder)
    @account  = account
    @data     = folder
    @num_msgs = {}
    @threads  = nil
    @messages = nil
    count_msgs()
  end

  def expunge
    @account.expunge
  end

  def get_messages
    @account.get_messages
  end

  def name
    @data.name
  end

  def count_msgs
    @num_msgs = @account.num_msgs(name)
  end

  def stats_to_s
    if num_msgs["RECENT"].to_i == 0 && num_msgs["MESSAGES"].to_i == 0
      return ""
    else
      return "%s %5s" % [num_msgs["RECENT"].to_s, num_msgs["MESSAGES"].to_s]
    end
  end

  def to_s
    name.gsub(/[^\.]+\./," ")
  end
end

