#!/usr/bin/env ruby
#
# emai - a Ncurses based IMAP Client
# Copyright Andreas Hauser, buggs-emai@splashground.de, 2003, 2004, 2005
#

require 'logger'
require 'frontend/emaifrontend'

def read_accounts
  accounts = {}
  file = "#{ENV['HOME']}/.emai/accounts.gpg"

  eval(`gpg -d #{file}`)
  raise "Wrong password? Or maybe no #{file}?" if accounts == {}

  return accounts
end

#
# MAIN
#

$debug = 0
err_file = "#{ENV['HOME']}/.emai/err"
$logger = Logger.new(err_file, 5, 1024000)

Signal.trap("TERM") do
  @frontend.shutdown() if @frontend
  $stderr.puts "caught SIGTERM: Terminating ..."
  exit(1)
end

Signal.trap("INT") do
  @frontend.shutdown() if @frontend
  $stderr.puts "caught SIGINT: Terminating ..."
  exit(1)
end

# BSD only
#Signal.trap("INFO") do
#  $stderr.print File.open(err_file) {|f| f.readlines[-1]}
#end

begin
  @accounts = read_accounts()
  @frontend = EmaiFrontend.new
  @frontend.accounts = @accounts
  @frontend.init_curses()
  @frontend.start()
  @frontend.looping()
  @frontend.shutdown()
rescue
  @frontend.shutdown() if @frontend
  $stderr.puts $!
  $stderr.print File.open(err_file) {|f| f.readlines[-5,-1]}
  exit(1)
end

# vim: ts=2 sw=2 expandtab
