#
# Module to send/reply/edit messages
#

require 'email/tempfile'

module EmaiEmail
  def default_header
    "User-Agent: Emai/0.0.3\n" +
    "Mime-Version: 1.0\n" +
    'Content-Type: text/plain; charset="UTF-8"' + "\n" +
    "Content-Disposition: inline\n"
  end

  def edit(mail)
    tmpfile = ""
    Tempfile.open("emai-", nil, ".eml") { |f|
      tmpfile = f.path
      f.puts mail
      f.flush
      @display_mutex.synchronize do
        Ncurses.def_prog_mode            # save current tty modes
        Ncurses.endwin                   # restore original tty modes
        system("$EDITOR #{tmpfile}")
      end
    }
    return tmpfile
  end

  def send_mail(sendmail, tmpfile)
    if FileTest.readable?(tmpfile)
      system(sendmail + " < " + tmpfile)
    else
      raise "Email::Send::Error"
    end
  end

  def forward_email(message, account)
    reply_mail(message, account, "", true)
  end

  def list_reply_mail(message, account)
    extra_header = ""
    reply_mail(message, account, extra_header, false, true)
  end

  def group_reply_mail(message, account)
    cc = []
    
    cc = message.envelope.cc.collect { |a|
      next if "#{a.mailbox}@#{a.host}" == account.conf["emailaddress"]
      "#{a.name} <#{a.mailbox}@#{a.host}>"
    } if message.envelope.cc
    
    cc += message.envelope.to.collect { |a|
      next if "#{a.mailbox}@#{a.host}" == account.conf["emailaddress"]
      "#{a.name} <#{a.mailbox}@#{a.host}>"
    } if message.envelope.to

    cc.uniq!
    extra_header = "CC: " + cc.join(",") + "\n" if cc.length > 0
    reply_mail(message, account, extra_header)
  end
  
  def reply_mail(message, account, extra_header="", forwarded=false, listreply=false)
    emailaddress   = account.conf["emailaddress"]
    sendmail       = account.conf["sendmail"]
    x_extra_header = account.conf["x-extraheader"]
    emailname      = account.conf["emailname"]
    extra_header  += "BCC: #{account.conf["BCC"]}\n" if not(listreply) && account.conf["BCC"]
    body           = message.decoded_body
    header         = message.header
    envelope       = message.envelope

    # read as x_extra_header.each_pair
    0.step(x_extra_header.length - 2, 2) { |i|
      extra_header += "#{x_extra_header[i]}: #{x_extra_header[i + 1]}\n"
    } if x_extra_header && x_extra_header.length.modulo(2) == 0
    
    to_from = nil
    to      = ""
    writer  = "someone"
    date    = "someday in the 3rd millenium"
    subject = header.fields["Subject"].strip
    subject = "Re: #{subject}" if subject !~ /(aw|re):/i

    if listreply && header.fields["List-Post"] then
      if match = /<mailto:([^>]+)>/.match(header.fields["List-Post"]) then
        to_from = match[1]
      else
        to_from = header.fields["List-Post"]
      end
      to = to_from
    else
      if envelope.reply_to
        to_from = envelope.reply_to[0]
      elsif envelope.from
        to_from = envelope.from[0]
      end
      to = "#{to_from.name} <#{to_from.mailbox}@#{to_from.host}>" if not(forwarded) && to_from
    end

    writer = envelope.from[0].mailbox if envelope.from[0]
    date   = envelope.date if envelope.date
    references  = [envelope.in_reply_to, envelope.message_id].compact.join(" ")
    in_reply_to = envelope.message_id

    if forwarded
      quote_intro = "\n---- Forwarded Message from #{writer} @ #{date} ----\n"
    else
      quote_intro = "\n#{writer} wrote @ #{date}:\n"
    end
      
    references  = "References: #{references}\n"
    reply_to    = "In-Reply-To: #{in_reply_to}\n"

    body_prepare(body)

    mail = default_header() +
           references +
           reply_to +
           "From: #{emailname} <#{emailaddress}>\n" +
           "Subject: #{subject}\n" +
           "To: #{to}\n" +
           extra_header +
           quote_intro +
           body

    tmpfile = edit(mail)
    send_mail(sendmail, tmpfile)
  end

  def body_prepare(body)
    body.gsub!(/\r/,"")
    body.gsub!(/\n-- \n.*/,"")
    body.gsub!(/^/,"> ")
  end

  def write_email(account)
    emailaddress   = account.conf["emailaddress"]
    sendmail       = account.conf["sendmail"]
    x_extra_header = account.conf["x-extraheader"]
    emailname      = account.conf["emailname"]
    extra_header   = ""
    extra_header   = "BCC: #{account.conf["BCC"]}\n" if account.conf["BCC"]
    mail = default_header() +
           "From: #{emailname} <#{emailaddress}>\n" +
           "Subject: \n" +
           "To: \n" +
           extra_header + "\n\n"
    tmpfile = edit(mail)
    send_mail(sendmail,tmpfile)
  end

end

# vim: ts=2 sw=2 expandtab
